' Windows Script Host Sample Script
'
' ------------------------------------------------------------------------
'               Copyright (C) Software Bisque
'
' ------------------------------------------------------------------------
Option Explicit

Dim L_Welcome_MsgBox_Message_Text
Dim L_Welcome_MsgBox_Title_Text
Dim L1

Dim CRLF
CRLF = Chr(13) & Chr(10)

L1 = "This script demonstrates how to map your telescope in an automated fashion.  For best results, you need a video camera, CCDSoft's video camera plug in (>=5.00.042), CCDSoft's Direct Guide turned on (>=5.00.014) if you have a Paramount; otherwise centering via camera relays is very time consuming."
L1 = L1 + CRLF + CRLF 
L1 = L1 + "See the function InitGlobalUserVariables() to set the path to the map list text file."

L_Welcome_MsgBox_Message_Text    = L1
L_Welcome_MsgBox_Title_Text      = "Windows Scripting Host Sample"


'Global Objects
Dim objTheSky
Dim objTele
Dim objCam

'Global User Variables see InitGlobalUserVariables()
Dim szPathToMapFile
Dim bIgnoreErrors


' ********************************************************************************
' *
' * Below is the flow of program execution
' * See the subroutine TargetLoop to see where the real work is done

Call Welcome()
Call InitGlobalUserVariables()
Call CreateObjects()
Call ConnectObjects()

Call TargetLoop()

Call DisconnectObjects()
Call DeleteObjects()

MsgBox("AutoMapViaVideo is complete.")


' ********************************************************************************
' *
' * Below are all the subroutines used in this sample
' *
Sub Welcome()
    Dim intDoIt

    intDoIt =  MsgBox(L_Welcome_MsgBox_Message_Text, _
                      vbOKCancel + vbInformation,    _
                      L_Welcome_MsgBox_Title_Text )
    If intDoIt = vbCancel Then
        WScript.Quit
    End If
End Sub

Sub InitGlobalUserVariables()

	'Use TheSky to generate a text file of mapping points
	szPathToMapFile = "d:\sky32\user\Exported User Data\Map.txt"

	'If you want your script to run all night regardless of errors, set bIgnoreErrors = True
	bIgnoreErrors = False

End Sub

Sub GetCoordinatesFromLine(LineFromFile, ObjName, dRa, dDec)

	ObjName = Mid(LineFromFile,1,15)
	dRa		= Mid(LineFromFile,22,13)
	dDec	= Mid(LineFromFile,40,12)

End Sub


Sub PromptOnError(bErrorOccurred)
	Dim bExitScript

	'Debugging-remove the single quote from the line below
	'Exit Sub
		
	bErrorOccurred = False
	bExitScript = False

	if (bIgnoreErrors = True) then 
		'Ignore all errors except when the user Aborts
		if (CStr(Hex(Err.Number)) = "800404BC") then 
			'Do nothing and let the user abort
		else
			Err.Clear
		end if
	end if

	if (Err.Number) then 
		bErrorOccurred = True
		bExitScript = MsgBox ("An error has occurred running this script.  Error # " & CStr(Hex(Err.Number)) & " " & Err.Description + CRLF + CRLF + "Exit Script?", vbYesNo + vbInformation)
	end if

	If bExitScript = vbYes Then
		WScript.Quit
	End if 

End Sub


Sub MapIt()

	Dim objMap

	Set objMap = CreateObject("Sky.TheSky.1")

	objMap.Map

	Set objMap = Nothing

End Sub

Sub TargetLoop()

	'If things get really bad, comment out the line below to have WSH tell 
	'the line number of the error
	On Error Resume Next

	Dim MyFile
	Dim fso
	Const ForReading = 1
	Dim dRa
	Dim dDec
	Dim szObjName
	Dim bErrorOccurred

	Set fso = CreateObject("Scripting.FileSystemObject")
	Set MyFile = fso.OpenTextFile(szPathToMapFile, ForReading)

	Call PromptOnError(bErrorOccurred)

	Do While (MyFile.AtEndOfStream <> True)

		Err.Clear 'Clear the error object
		bErrorOccurred = FALSE 'No error has occurred

		Call GetCoordinatesFromLine(MyFile.ReadLine, szObjName, dRa, dDec)

		Call PromptOnError(bErrorOccurred)

		if (bErrorOccurred = False) then
			'Identify the object in TheSky
			Call objTheSky.GetObjectRaDec(szObjName)
			Call objTele.SlewToRaDec(objTheSky.dObjectRa, objTheSky.dObjectDec, szObjName)
			Call PromptOnError(bErrorOccurred)
		end if

		if (bErrorOccurred = False) then
			objCam.PropDbl("m_dExposureTime")	= 0
			objCam.lAsynchronous = 0
			Call objCam.TakeImage()
			Call PromptOnError(bErrorOccurred)
		end if

		if (bErrorOccurred = False) then
			objCam.CenterBrightestObject
			Call PromptOnError(bErrorOccurred)
		end if

		if (bErrorOccurred = False) then
			'Make sure TheSky's server settings allow remote clients to map
			MapIt
			Call PromptOnError(bErrorOccurred)
		end if
	Loop

End Sub


Sub CreateObjects()
	Set objTheSky = WScript.CreateObject("RASCOM.RASCOMTheSky.1")
	Set objTele = WScript.CreateObject("RASCOM.RASCOMTele.1")
	Set objCam = WScript.CreateObject("CCDSoft.Camera.1")
End Sub

Sub ConnectObjects()
	objTheSky.Connect()
	objTele.Connect()
	objCam.Connect()
End Sub

Sub DisconnectObjects()
	objTheSky.Disconnect()
	objTele.Disconnect()
	objCam.Disconnect()
End Sub 

Sub DeleteObjects()
	Set objTheSky = Nothing
	Set objTele = Nothing
	Set objCam = Nothing
End Sub 

	
