' Windows Script Host Sample Script
'
' ------------------------------------------------------------------------
'               Copyright (C) Software Bisque
'
' ------------------------------------------------------------------------
Option Explicit

Dim L_Welcome_MsgBox_Message_Text
Dim L_Welcome_MsgBox_Title_Text

L_Welcome_MsgBox_Message_Text    = "This script demonstrates how to synch a telescope based upon an astrometric solution of an image."
L_Welcome_MsgBox_Title_Text      = "Windows Scripting Host Sample"

'Global Objects
Dim objTheSky
Dim objTele
Dim objCam

'Global User Variables see InitGlobalUserVariables()
Dim bLX200
Dim dImageScale


' ********************************************************************************
' *
' * Below is the flow of program execution
' * See the subroutine TargetLoop to see where the real work is done

Call Welcome()
Call InitGlobalUserVariables()
Call CreateObjects()
Call ConnectObjects()

Call TargetLoop()

Call DisconnectObjects()
Call DeleteObjects()


' ********************************************************************************
' *
' * Below are all the subroutines used in this sample
' *
Sub Welcome()
    Dim intDoIt

    intDoIt =  MsgBox(L_Welcome_MsgBox_Message_Text, _
                      vbOKCancel + vbInformation,    _
                      L_Welcome_MsgBox_Title_Text )
    If intDoIt = vbCancel Then
        WScript.Quit
    End If
End Sub

Sub InitGlobalUserVariables()

	'If bLX200 is True, the TargetLoop() attempts to make the 
	'telescope point better by taking an image, getting an astrometric solution, 
	'resynchronizing the telescope based on the astrometric solution, and slewing 
	'to the desired object again.
	'For Paramount telescope mounts, set this value to false
	bLX200 = True

	'Set your image scale in arc seconds per pixel
	'If you don't know your image scale, do a manual ImageLink in TheSky
	dImageScale = 1.77

End Sub

Sub TargetLoop()
	'On Error Resume Next
	Dim i
	Dim Target

	For i=1 To 26
		Target = GetTarget(i)

		if (bLX200 = True) then
			Call objTheSky.GetObjectRaDec(Target)

			Call objTele.SlewToRaDec(objTheSky.dObjectRa, objTheSky.dObjectDec, Target)

			if (SkipToNextObject()) then
				Next
			endif
			
			objCam.dExposureTime = 1.0
			Call objCam.TakePicture()

			'Perform an image link with the last picture
			Call objTheSky.ImageLink(dImageScale, objTheSky.dObjectRa, objTheSky.dObjectDec)

			'Get the ra dec from ImageLink's astrometric solution
			Call objTheSky.GetObjectRaDec("Image Link Information")

			'Sync the telescope to where its actually pointing
			'Security settings in TheSky must allow remote clients to sync the telescope
			'or this call will fail with permission denied
			Call objTele.Sync(objTheSky.dObjectRa,objTheSky.dObjectDec,Target)
		end if

		'Slew to the desired target 
		Call objTheSky.GetObjectRaDec(Target)
		Call objTele.SlewToRaDec(objTheSky.dObjectRa, objTheSky.dObjectDec, Target)

		'Take the desired exposure
		objCam.dExposureTime = 3.0
		Call objCam.TakePicture()
	Next

End Sub

Sub CreateObjects()
	Set objTheSky = WScript.CreateObject("RASCOM.RASCOMTheSky.1")
	Set objTele = WScript.CreateObject("RASCOM.RASCOMTele.1")
	Set objCam = WScript.CreateObject("RASCOM.RASCOMCam.1")
End Sub

Sub ConnectObjects()
	objTheSky.Connect()
	objTele.Connect()
	objCam.Connect()
End Sub

Sub DisconnectObjects()
	objTheSky.Disconnect()
	objTele.Disconnect()
	objCam.Disconnect()
End Sub 

Sub DeleteObjects()
	Set objTheSky = Nothing
	Set objTele = Nothing
	Set objCam = Nothing
End Sub 

Function GetTarget(Index)
	Dim T

	Select Case Index
		Case 1	T="NGC1348"
		Case 2	T="NGC1491"
		Case 3	T="NGC1708"
		Case 4	T="NGC1790"
		Case 5	T="NGC1798"
		Case 6	T="NGC2165"
		Case 7	T="NGC2334"
		Case 8	T="NGC2436"
		Case 9	T="NGC2519"
		Case 10	T="NGC2605"
		Case 11	T="NGC2689"
		Case 12	T="NGC2666"
		Case 13	T="NGC4381"
		Case 14	T="NGC5785"
		Case 15	T="NGC5804"
		Case 16	T="NGC6895"
		Case 17	T="NGC6991"
		Case 18	T="NGC7011"
		Case 19 T="NGC7058"
		Case 20	T="M39"
		Case 21	T="NGC7071"
		Case 22	T="NGC7150"
		Case 23	T="NGC7295"
		Case 24	T="NGC7394"
		Case 25	T="NGC7686"
		Case 26	T="NGC7801"
	End Select

	GetTarget = T

End Function
	
'This function takes normally fatal errors
'
Function SkipToNextObject()
	Dim bClearIt
	Dim bExitScript

	bClearIt = False

	SkipToNextObject = False

	'if (Err.Number) then
	'	Select Case Err.Number
	'		'Image link failures are cleared
	'		case 0x80040675 bClearIt = True
	'	end Select
	'end if

	if (bClearIt)
		Err.Clear
		SkipToNextObject = True
	else
		bExitScript = MsgBox ("Error # " & CStr(Err.Number) & " " & Err.Description & "  Exit Script?", vbYesNo + vbInformation)

		If bExitScript = vbYes Then
			WScript.Quit
		end if 
    End If

End Function