' Windows Script Host Sample Script
'
' ------------------------------------------------------------------------
'               Copyright (C) Software Bisque
'
' ------------------------------------------------------------------------
Option Explicit

Dim L_Welcome_MsgBox_Message_Text
Dim L_Welcome_MsgBox_Title_Text
Dim L1

Dim CRLF
CRLF = Chr(13) & Chr(10)

'This script differs from ImageLinkAndSync.vbs in that
'it uses CCDSoft V5 object model, adds an optional delay value
'and prefixes all pointing image filenames with a user definable prefix.
'The prefix allows easy differentiation from pointing vs real images

L1 = "This script demonstrates how to acquire images from a list of targets in "
L1 = L1 + "a text file.  Moreover, the script can accommodate a poor pointing telescope."
L1 = L1 + "   This is accomplished by taking two images.  The first image is used to "
L1 = L1 + "produce an Image Link astrometric solution to determine precisely where "
L1 = L1 + "the telescope 'actually' ended up pointing.  The telescope is re-synced "
L1 = L1 + "based upon the astrometric solution and the telescope is slewed to the "
L1 = L1 + "desired target again.  Finally, the desired picture is acquired."
L1 = L1 + CRLF + CRLF
L1 = L1 + "An undesirable result of the above method of telescope pointing correction "
L1 = L1 + "is that it roughly cuts in half data acquisition.  Software Bisque "
L1 = L1 + "maintains that the correct and rigorous approach to good telescope "
L1 = L1 + "pointing is through solid telescope hardware like the Paramount "
L1 = L1 + "GT-1100 and through telescope modeling with TPoint to account for "
L1 = L1 + "inevitable system errors like offset error, polar misalignment, "
L1 = L1 + "non-perpendicular axis, fork flexure, tube flexure, etc."



L_Welcome_MsgBox_Message_Text    = L1
L_Welcome_MsgBox_Title_Text      = "Windows Scripting Host Sample"


'Global Objects
Dim objTheSky
Dim objTele
Dim objCam

'Global User Variables see InitGlobalUserVariables()
Dim szPathToMapFile
Dim dImageScale
Dim dExposure1
Dim dExposure2
Dim szPointingImagePrefix
Dim dDelay


' ********************************************************************************
' *
' * Below is the flow of program execution
' * See the subroutine TargetLoop to for where the real work is done

Call Welcome()
Call InitGlobalUserVariables()
Call CreateObjects()
Call ConnectObjects()
Call TargetLoop()
Call DisconnectObjects()
Call DeleteObjects()


' ********************************************************************************
' *
' * Below are all the subroutines used in this sample
' *
Sub Welcome()
    Dim intDoIt

    intDoIt =  MsgBox(L_Welcome_MsgBox_Message_Text, _
                      vbOKCancel + vbInformation,    _
                      L_Welcome_MsgBox_Title_Text )
    If intDoIt = vbCancel Then
        WScript.Quit
    End If
End Sub

Sub InitGlobalUserVariables()

	'Use TheSky's Data Export command to quickly and easily 
	'generate a text file of target objects
	'The default is below
	'szPathToMapFile = "C:\Program Files\Software Bisque\TheSky\User\Exported Data\Target.txt"
	szPathToMapFile = "D:\Sky32\User\Exported User Data\Target.txt"

	'Enter your image scale here in arc seconds per pixel
	'Do an ImageLink by hand once to determine your scale
	dImageScale = 1.7

	'Enter the length of the exposure for the doing the Image Link astrometric
	'solution, typically this is less than the dExposure2 and around 10 seconds
	dExposure1 = 1.0

	'Enter the length of the exposure for the desired image
	dExposure2 = 2.0

	'This prefix will be appended to the front of the image file names that
	'were acquired for pointing.  This will allow easy differentiation
	szPointingImagePrefix = "Pointing Image"

	'Enter an amount to delay before each image is acquired
	dDelay = 0

End Sub


Sub GetDataFromTextFile(LineFromFile, szTargetName, dRa, dDec)

	szTargetName= Mid(LineFromFile,1,18)
	dRa			= CDbl(Mid(LineFromFile,45,11))
	dDec		= CDbl(Mid(LineFromFile,61,11))

End Sub


Sub PromptOnError(bErrorOccurred)

	Dim bExitScript

	bErrorOccurred = False
	bExitScript = False

	if (Err.Number) then 
		bErrorOccurred = True
		bExitScript = MsgBox ("An error has occurred running this script.  Error # " & CStr(Hex(Err.Number)) & " " & Err.Description + CRLF + CRLF + "Exit Script?", vbYesNo + vbInformation)
	end if

	If bExitScript = vbYes Then
		DisconnectObjects()
		DeleteObjects()
		WScript.Quit
	End if 

End Sub

Sub TargetLoop()
	On Error Resume Next
	Dim TxtFile
	Dim fso
	Const ForReading = 1
	Dim dRa
	Dim dDec
	Dim bErrorOccurred
	Dim szTargetName

	Set fso = CreateObject("Scripting.FileSystemObject")
	Set TxtFile = fso.OpenTextFile(szPathToMapFile, ForReading)

	Call PromptOnError(bErrorOccurred)

	Do While (TxtFile.AtEndOfStream <> True)

		Err.Clear 'Clear the error object
		bErrorOccurred = FALSE 'No error has occurred

		Call GetDataFromTextFile(TxtFile.ReadLine, szTargetName, dRa, dDec)

		if (bErrorOccurred = False) then
			Call objTele.SlewToRaDec(dRa, dDec, szTargetName)
			Call PromptOnError(bErrorOccurred)
		end if
		
		if (bErrorOccurred = False) then
			objCam.PropDbl("m_dExposureTime")		= dExposure1
			objCam.PropStr("m_csAutoSavePrefix")	= szPointingImagePrefix 
			Call objCam.TakeImage()
			Call PromptOnError(bErrorOccurred)
		end if

		'Perform an image link with the above picture
		'Make sure CCDSoft Server settings have "Remote clients use Orchestrate's Auto-Map capabilites" turned on
		if (bErrorOccurred = False) then
			Call objTheSky.ImageLink(dImageScale, dRa, dDec)
			Call PromptOnError(bErrorOccurred)
		end if

		'Get the ra dec from ImageLink's astrometric solution
		if (bErrorOccurred = False) then
			Call objTheSky.GetObjectRaDec("Image Link Information")
			Call PromptOnError(bErrorOccurred)
		end if

		'Sync the telescope to where its actually pointing
		'Security settings in TheSky must allow remote clients to sync the telescope
		'or this call will fail with permission denied
		if (bErrorOccurred = False) then
			Call objTele.Sync(objTheSky.dObjectRa,objTheSky.dObjectDec, "Image Link's Sync")
			Call PromptOnError(bErrorOccurred)
		end if
	
		'Slew to the desired target again, now that we've "fixed" poor pointing
		if (bErrorOccurred = False) then
			Call objTele.SlewToRaDec(dRa, dDec, szTargetName)
			Call PromptOnError(bErrorOccurred)
		end if

		'Take the desired exposure
		if (bErrorOccurred = False) then
			objCam.PropDbl("m_dExposureTime")		= dExposure2
			objCam.PropStr("m_csAutoSavePrefix")	= ""
			Call objCam.TakeImage()
			Call PromptOnError(bErrorOccurred)
		end if

	Loop

End Sub


Sub CreateObjects()
	Set objTheSky = WScript.CreateObject("RASCOM.RASCOMTheSky.1")
	Set objTele = WScript.CreateObject("RASCOM.RASCOMTele.1")
	Set objCam = WScript.CreateObject("CCDsoft.Camera.1")

End Sub

Sub ConnectObjects()
	objTheSky.Connect()
	objTele.Connect()
	objCam.Connect()
	objCam.PropLng("m_bUseSubFrame") = 0'Make sure UI has not turned on subframe
	objCam.PropDbl("m_dDelay")			 = dDelay
End Sub

Sub DisconnectObjects()
	objTheSky.Disconnect()
	objTele.Disconnect()

	objCam.PropStr("m_csAutoSavePrefix")		= ""
	objCam.PropDbl("m_dDelay")					= 0
	objCam.Disconnect()

End Sub 

Sub DeleteObjects()
	Set objTheSky = Nothing
	Set objTele = Nothing
	Set objCam = Nothing
End Sub 

	
