' Windows Script Host Sample Script
'
' ------------------------------------------------------------------------
'               Copyright (C) Software Bisque
'
' ------------------------------------------------------------------------
Option Explicit

Dim L_Welcome_MsgBox_Message_Text
Dim L_Welcome_MsgBox_Title_Text
Dim L1

Dim CRLF
CRLF = Chr(13) & Chr(10)

L1 = "This script demonstrates one way to search for possible supernova.  "
L1 = L1 + CRLF + CRLF 
L1 = L1 + "Using TheSky's Data Export command, a list of candidate galaxies is generated and saved to a text file.  "
L1 = L1 + "This script reads the objects from the text file, slews to each object and acquires an image.  "
L1 = L1 + "The list can be optimized for a German Equatorial Mount and so that the images are taken near the zenith."
L1 = L1 + CRLF + CRLF 
L1 = L1 + "See the function InitGlobalUserVariables() to enter your desired exposure time and to set the path to the object list text file."

L_Welcome_MsgBox_Message_Text    = L1
L_Welcome_MsgBox_Title_Text      = "Windows Scripting Host Sample"

'FYI Any line preceded by a single quote is ignored and not part of the script

'Global Objects
Dim objTheSky
Dim objTele
Dim objCam

'Global User Variables see InitGlobalUserVariables()
Dim szPathToTargetFile
Dim dExposure
Dim bIgnoreErrors


' ********************************************************************************
' *
' * Below is the flow of program execution
' * See the subroutine TargetLoop to for where the real work is done

Call Welcome()
Call InitGlobalUserVariables()
Call CreateObjects()
Call ConnectObjects()
Call TargetLoop()
Call DisconnectObjects()
Call DeleteObjects()


' ********************************************************************************
' *
' * Below are all the subroutines used in this sample
' *
Sub Welcome()
    Dim intDoIt

    intDoIt =  MsgBox(L_Welcome_MsgBox_Message_Text, _
                      vbOKCancel + vbInformation,    _
                      L_Welcome_MsgBox_Title_Text )
    If intDoIt = vbCancel Then
        WScript.Quit
    End If
End Sub

Sub InitGlobalUserVariables()

	'Use TheSky's Data Export command to quickly and easily 
	'generate a text file of target objects
	'The default is below
	'szPathToTargetFile = "C:\Program Files\Software Bisque\TheSky\User\Exported Data\Target.txt"
	szPathToTargetFile = "E:\Sky32\User\Exported Data\SupernovaSearch.txt"

	'Enter the length of the exposure for the desired image
	'dExposure = 60.0
	dExposure = 1.0
	
	'If you want your script to run all night regardless of errors, set bIgnoreErrors = True
	bIgnoreErrors = False

End Sub


Sub GetDataFromTextFile(LineFromFile, szTargetName, dRa, dDec)

	szTargetName= Mid(LineFromFile,1,18)
	dRa			= CDbl(Mid(LineFromFile,45,11))
	dDec		= CDbl(Mid(LineFromFile,61,11))

End Sub


Sub PromptOnError(bErrorOccurred)

	Dim bExitScript
	
	bErrorOccurred = False
	bExitScript = False

	if (bIgnoreErrors = True) then 
		'Ignore all errors except when the user Aborts
		if (CStr(Hex(Err.Number)) = "800404BC") then 
			'Do nothing and let the user abort
		else
			Err.Clear
		end if
	end if

	if (Err.Number) then 
		bErrorOccurred = True
		bExitScript = MsgBox ("An error has occurred running this script.  Error # " & CStr(Hex(Err.Number)) & " " & Err.Description + CRLF + CRLF + "Exit Script?", vbYesNo + vbInformation)
	end if

	If bExitScript = vbYes Then
		WScript.Quit
	End if 

End Sub

Sub TargetLoop()
	On Error Resume Next
	Dim TxtFile
	Dim fso
	Const ForReading = 1
	Dim dRa
	Dim dDec
	Dim bErrorOccurred
	Dim szTargetName

	Set fso = CreateObject("Scripting.FileSystemObject")
	Set TxtFile = fso.OpenTextFile(szPathToTargetFile, ForReading)

	Do While (TxtFile.AtEndOfStream <> True)

		Err.Clear 'Clear the error object
		bErrorOccurred = FALSE 'No error has occurred
		
		Call GetDataFromTextFile(TxtFile.ReadLine, szTargetName, dRa, dDec)

		if (bErrorOccurred = False) then
			Call objTele.SlewToRaDec(dRa, dDec, szTargetName)
			Call PromptOnError(bErrorOccurred)
		end if
		
		if (bErrorOccurred = False) then
			objCam.dExposureTime = dExposure
			Call objCam.TakePicture()
			Call PromptOnError(bErrorOccurred)
		end if

	Loop

End Sub


Sub CreateObjects()
	Set objTheSky = WScript.CreateObject("RASCOM.RASCOMTheSky.1")
	Set objTele = WScript.CreateObject("RASCOM.RASCOMTele.1")
	Set objCam = WScript.CreateObject("RASCOM.RASCOMCam.1")
End Sub

Sub ConnectObjects()
	objTheSky.Connect()
	objTele.Connect()
	objCam.Connect()
End Sub

Sub DisconnectObjects()
	objTheSky.Disconnect()
	objTele.Disconnect()
	objCam.Disconnect()
End Sub 

Sub DeleteObjects()
	Set objTheSky = Nothing
	Set objTele = Nothing
	Set objCam = Nothing
End Sub 

	
