'From Douglas Long and Peter McCullough of the XO Project
'Code written by Jeff Stys and updated for TheSky6

Const code_version = "1.1.0"
' Pathnames changed to run on rack computers 
' RASCOM object calls changed to use TheSky6

Dim objTele
Dim objGlobal

'Create objects
Set objGlobal = New globals
Set objTele = WScript.CreateObject("TheSky6.RASCOMTele")

'Load global variables from file
'##### HARDCODED PATH! ######'
Call objGlobal.load("C:\Documents and Settings\Administrator\Desktop\MountControl\mount_globals.txt")

Call log(objGlobal.logpath,"### DOY = " & datepart("y",now()) & " (actual)")
Call log(objGlobal.logpath,"### Version: " & code_version)
Call log(objGlobal.logpath,"")
Call objGlobal.printToLog()
Call log(objGlobal.logpath,"")
If objGlobal.zoamin > objGlobal.zoamax Then
  Call log(objGlobal.logpath,"ERROR: zoamin > zoamax.  Ending Program!")
  Call log(objGlobal.logpath,"")
  Call log(objGlobal.logpath,"")
  WScript.Quit
End If

'Initialize communications with the telescope mount
Call initialize()

Call observe()

'End of night.  Safely park & disconnect the mount
Call stow()


'*********************************************************************
'** function loadcommands(filename, commandarray) ********************
'*********************************************************************
' filename -- path to observing strategy data file
'             ex:  "c:\2003.txt"
' commandarray must be a static array defined before using
'   it is a call by reference parameter
' Only loads commands for current day of year (DOY)
' After midnight but before 4pm uses DOY of previous evening
' Returns the number of commands stored in commandarray, 0 if error

function loadcommands(filename, commandarray)
  dim filesys, readfile
  set filesys = CreateObject("Scripting.FileSystemObject")

  If filesys.FileExists(filename) Then
    Call log(objGlobal.logpath,"Strategy file exists: " & filename)
    set readfile = filesys.OpenTextFile(filename)

    today = CInt(datepart("y",now()))
    If time < objGlobal.doychangetime Then
      today = CInt(datepart("y",dateadd("d",-1,now())))
      Call log(objGlobal.logpath,"WARNING: Time is after midnight, and before " & objGlobal.doychangetime & ".  Using previous SDOY: " & today)
    End If

    If readfile.AtEndOfStream Then
      readfile.close()
      loadcommands = 0
      Call log(objGlobal.logpath,"ERROR: Strategy file is EMPTY.")
    Else
      index = 0
      do while readfile.AtEndOfStream eqv FALSE
        line = readfile.ReadLine()
        words = split(line," ")
        num_elements = 0
        for each word in words
          num_elements = num_elements+1
        next

        If left(line,1) <> "#" and line <> "" Then
          If num_elements <> 7 Then
            Call log(objGlobal.logpath,"ERROR: Skipping invalid strategy line:")
            Call log(objGlobal.logpath,"  " & line)
          Else
            'words = Split(line," ")
            doy = CInt(words(0))
            if doy = today then
              commandarray(index) = line
              index = index + 1
            End If
          End If
        End If
      Loop
      readfile.close()
  
      loadcommands = index
    End If
  Else
    loadcommands = 0
    Call log(objGlobal.logpath,"ERROR: Strategy file does NOT exist: " & filename)
  End If
  Call log(objGlobal.logpath,"")
end function 'loadcommands





'*********************************************************************
'*** decscan(ra,decmin,decmax,decrate,duration) **********************
'*********************************************************************


function decscan(ra,decmin,decmax,decrate,duration)
  Call log(objGlobal.logpath,"Begin drift scan loop.")
  Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
  Call objtele.settracking(1,0,0,0)
  thetime = now()
  waitsecs = syncsecs(thetime,objGlobal.syncmultiple)
  scanstart = dateadd("s",waitsecs,thetime)
  scanend = dateadd("s",duration,scanstart)
  ha_start = ra2ha(lst(objGlobal.longitude,scanstart,objGlobal.timezone), ra)
  ha_end = ra2ha(lst(objGlobal.longitude,scanend,objGlobal.timezone), ra)
  
  Call log(objGlobal.logpath,"Check if mount is authorized to operate.")
  isauth = auth(objGlobal.authpath)
  Call log(objGlobal.logpath,"Mount authorization boolean: " & isauth)

  if isauth = 0 Then
    Call log(objGlobal.logpath,"WARNING: Mount not authorized to operate.")
    Call log(objGlobal.logpath,"Initiate mount homing.")
    call objtele.findhome()
    Call log(objGlobal.logpath,"Mount finished homing.")
    waitsecs = syncsecs(now(),objGlobal.syncmultiple)
    If waitsecs < duration Then
      waitsecs = waitsecs + duration
    Else
      waitsecs = waitsecs - objGlobal.slewtime
    End If
    Call log(objGlobal.logpath,"Wait for " & waitsecs & " seconds. (" & dateadd("s",waitsecs,time) & ")")
    Wscript.sleep waitsecs*1000
  Elseif ha_start < objGlobal.haminlim or ha_start > objGlobal.hamaxlim or ha_end > objGlobal.hamaxlim or ha_end < objGlobal.haminlim or decmin < objGlobal.decminlim or decmin > objGlobal.decmaxlim or decmax > objGlobal.decmaxlim or decmax < objGlobal.decminlim Then  
    Call log(objGlobal.logpath,"ERROR: Mount not authorized to operate.  Next positon out of allowed range.")
    Call log(objGlobal.logpath,"  ha_start: " & ha_start & "  haminlim: " & objGlobal.haminlim)
    Call log(objGlobal.logpath,"  ha_end:   " & ha_end & "  hamaxlim: " & objGlobal.hamaxlim)
    Call log(objGlobal.logpath,"  decmin:   " & decmin & "  decminlim: " & objGlobal.decminlim)
    Call log(objGlobal.logpath,"  decmax:   " & decmax & "  decmaxlim: " & objGlobal.decmaxlim)
    'return error signal to calling function
    decscan = false
    exit function
  Else
    Call log(objGlobal.logpath,"Mount is authorized to operate.")
    Call log(objGlobal.logpath,"ha_start=" & ha_start & ", ha_end=" & ha_end)

    'beginning of scan in zone of avoidance.  wait
    If ha_start >= objGlobal.zoamin and ha_start <= objGlobal.zoamax then
      Call log(objGlobal.logpath,"ha_start is within meridian zone of avoidance.")
      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      Call objtele.settracking(1,0,0,0)
      Call log(objGlobal.logpath,"Initiate mount homing.")
      call objtele.findhome()
      Call log(objGlobal.logpath,"Mount finished homing.")
      'wait until out of zoa
      waitsecs = syncsecs(now(),objGlobal.syncmultiple)
      waitsecs = waitsecs + ((objGlobal.zoamax - ha_start)*3600)
      Call log(objGlobal.logpath,"Waiting for " & waitsecs & " seconds for zoa passage. (" & dateadd("s",waitsecs,time) & ")")
      Wscript.sleep waitsecs*1000
      Call log(objGlobal.logpath,"Done waiting.")

    'end of scan in zone of avoidance.  wait
    Elseif ha_end >= objGlobal.zoamin and ha_end <= objGlobal.zoamax then
      Call log(objGlobal.logpath,"ha_end is within meridian zone of avoidance.")
      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      Call objtele.settracking(1,0,0,0)
      Call log(objGlobal.logpath,"Initiate mount homing.")
      call objtele.findhome()
      Call log(objGlobal.logpath,"Mount finished homing.")
      'wait until out of zoa
      waitsecs = syncsecs(now(),objGlobal.syncmultiple)
      waitsecs = waitsecs + ((objGlobal.zoamax - ha_start)*3600)
      Call log(objGlobal.logpath,"Waiting for " & waitsecs & " seconds for zoa passage. (" & dateadd("s",waitsecs,time) & ")")
      Wscript.sleep waitsecs*1000
      Call log(objGlobal.logpath,"Done waiting.")

    'scan straddles zoa.  wait
    Elseif ha_start <= objGlobal.zoamin and ha_end >= objGlobal.zoamax then
      Call log(objGlobal.logpath,"Next scan straddles zone of avoidance.")
      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      Call objtele.settracking(1,0,0,0)
      Call log(objGlobal.logpath,"Initiate mount homing.")
      call objtele.findhome()
      Call log(objGlobal.logpath,"Mount finished homing.")
      'wait until out of zoa, roughly in sync with normal cadence
      ''waitsecs = syncsecs(now(),objGlobal.syncmultiple)
      ''waitsecs = waitsecs + ((objGlobal.zoamax - ha_start)*3600)
      waitsecs = (objGlobal.zoamax - ha_start)*3600
      Call log(objGlobal.logpath,"Waiting for " & waitsecs & " seconds for zoa passage. (" & dateadd("s",waitsecs,time) & ")")
      Wscript.sleep waitsecs*1000
      Call log(objGlobal.logpath,"Done waiting.")

    'scan north
    Elseif ha_end < objGlobal.zoamin then
      Call log(objGlobal.logpath,"Scan direction will be NORTH.")
      Call log(objGlobal.logpath,"Track at sidereal rate.")
      Call objtele.settracking(1,0,0,0)
      Call log(objGlobal.logpath,"Slew to drift scan starting position:")
      Call log(objGlobal.logpath,"  RA: " & ra & "  Dec: " & decmin)
      call objtele.slewtoradec(ra,decmin,"north")
      Call log(objGlobal.logpath,"Slew complete.")
      waitsecs = syncsecs(now(),objGlobal.syncmultiple)
      Call log(objGlobal.logpath,"Wait for " & waitsecs & " seconds until next syncmultiple of " & objGlobal.syncmultiple & " minutes.")
      Wscript.sleep waitsecs*1000
      Call log(objGlobal.logpath,"Set scanning speed:")
      Call log(objGlobal.logpath,"  RA:  sidereal")
      Call log(objGlobal.logpath,"  Dec: " & decrate & " arcsec/sec")
      Call objtele.settracking(1,0,0,decrate)
      Call log(objGlobal.logpath,"DRIFT SCANNING for " & duration & " seconds. (Done at " & dateadd("s",duration,now()) & ")")
      Wscript.sleep duration*1000
      Call log(objGlobal.logpath,"Drift scan complete.")
      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      call objtele.settracking(1,0,0,0)

    'scan south
    Elseif ha_start >= objGlobal.zoamax Then
      If decrate > 0 Then
        decrate = -decrate
      End If
      Call log(objGlobal.logpath,"Scan direction will be SOUTH.")
      Call log(objGlobal.logpath,"Track at sidereal rate.")
      Call objtele.settracking(1,0,0,0)
      Call log(objGlobal.logpath,"Slew to drift scan starting position:")
      Call log(objGlobal.logpath,"  RA: " & ra & "  Dec: " & decmax)
      Call objtele.slewtoradec(ra,decmax,"south")
      Call log(objGlobal.logpath,"Slew complete.")
      waitsecs = syncsecs(now(),objGlobal.syncmultiple)
      Call log(objGlobal.logpath,"Wait for " & waitsecs & " seconds until next syncmultiple of " & objGlobal.syncmultiple & " minutes.")
      Wscript.sleep waitsecs*1000
      Call log(objGlobal.logpath,"Set scanning speed:")
      Call log(objGlobal.logpath,"  RA:  sidereal")
      Call log(objGlobal.logpath,"  Dec: " & decrate & " arcsec/sec")
      Call objtele.settracking(1,0,0,decrate)
      Call log(objGlobal.logpath,"DRIFT SCANNING for " & duration & " seconds. (Done at " & dateadd("s",duration,now()) & ")")
      Wscript.sleep duration*1000
      Call log(objGlobal.logpath,"Drift scan complete.")
      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      call objtele.settracking(1,0,0,0)

    else
      Call log(objGlobal.logpath,"ERROR: Invalid scan logic.")
      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      Call objtele.settracking(1,0,0,0)
      Call log(objGlobal.logpath,"Initiate mount homing.")
      call objtele.findhome()
      Call log(objGlobal.logpath,"Mount finished homing.")
      waitsecs = syncsecs(now(),objGlobal.syncmultiple)
      If waitsecs <= duration Then 
        waitsecs = waitsecs + duration
      End If
      Call log(objGlobal.logpath,"Waiting for " & waitsecs & " seconds. (" & dateadd("s",waitsecs,time) & ")")
      Wscript.sleep waitsecs*1000
      Call log(objGlobal.logpath,"Done waiting.")
    end if 'north/south/meridian checks

  end if 'safe to slew/scan (mount position limit checks, authorization check)

  Call log(objGlobal.logpath,"Set tracking to sidereal rate (just in case).")
  Call objtele.settracking(1,0,0,0)
  Call log(objGlobal.logpath,"End of drift scan loop.")
  'signal nominal completion to calling function
  decscan = true
end function 'decscan




'*********************************************************************
'** lst(longitude,datetime,timezone) *********************************
'*********************************************************************
' longitude -- decimal degrees ('+' east, '-' for west)
' datetime -- a date format accepted by datepart() ie. now()
' timezone -- hst = 10;  account for summer time as needed
' requires the 'julian' function
' PORTED FROM 'ct2lst.pro' IDL CODE

function lst(longitude,datetime,timezone)

  dim c(3)

  c(0) = 280.46061837
  c(1) = 360.98564736629
  c(2) = 0.000387933
  c(3) = 38710000.0

  datetime = dateadd("h",timezone,datetime)
  yr=datepart("yyyy",datetime)
  mn=datepart("m",datetime)
  d=datepart("d",datetime)
  minutes = datepart("n",datetime)
  seconds = datepart("s",datetime)
  'time in decimal hours (0-24)
  hr=datepart("h",datetime) + minutes/60.0 + seconds/3600.0

  jd = julian(yr,mn,d,hr)

  jd2000 = 2451545.0
  t0 = jd - jd2000
  t = t0\36525

  'compute GST in seconds.
  theta = c(0) + (c(1)*t0) + (t^2)*(c(2) - (t\c(3)))

  lst = (theta + CDbl(longitude))/Cdbl(15.0)
  lst = CDbl(lst) - (CDbl(lst) \ CDbl(24.0))*CDbl(24.0)

  if lst < 0 then
    lst = 24.0 + lst
  end if

end function 'lst





'*********************************************************************
'** julian(yyyy,m,d,hh) **********************************************
'*********************************************************************
' yyyy -- four digit year
' m -- month (1-12)
' d -- day of month (1-31 as appropriate)
' hh -- time in decimal hours.  1:30:45 PM = 13.5125
' PORTED FROM 'jdcnv.pro' IDL CODE

function julian(yyyy, m, d, hh)
  yyyy = CLng(yyyy)
  m = CLng(m)
  d = CLng(d)
  L = (m-14)\12
  julian = d - CLng(32075) + (CLng(1461)*(yyyy+CLng(4800)+L))\4 + (CLng(367)*(m - 2-L*12))\12 - 3*((yyyy+CLng(4900)+L)\100)\4
  julian = CDbl(julian) + (hh/24.0) - 0.5
end function




'*********************************************************************
'** function syncsecs(thetime,multiple) ************************
'*********************************************************************
' thetime -- timestamp (ie. now())
' multiple -- sync to minutes which are multiples of this num (in min)
' start -- must be 1 right now...
' returns number of seconds until next sync time

function syncsecs(thetime,multiple)
  'once offset can be other than 1, make this a parameter and global variable
  offset = 1
  max_syncmin = ((60\multiple)*multiple)+offset
  if max_syncmin >= 60 Then
    max_syncmin = max_syncmin - multiple
  end if
  hr = hour(thetime)
  min = minute(thetime)
  sec = second(thetime)
  sec1 = sec + (60*min) + (3600*hr)
  If (min + multiple + offset) >= 60 Then
    If min = max_syncmin and sec = 0 Then
      nexttime = thetime
    ElseIf min < max_syncmin Then
      nexttime = timeserial(hr,max_syncmin,0)
    ElseIf min mod multiple = offset and sec = 0 Then
      nexttime = thetime
    Else
      nexttime = timeserial(hr+1,offset,0)
    End If
    sec2 = datepart("s",nexttime) + (60*datepart("n",nexttime)) + (3600*datepart("h",nexttime))
  ElseIf min mod multiple = offset and sec = 0 Then
    nexttime = thetime
    sec2 = datepart("s",nexttime) + (60*datepart("n",nexttime)) + (3600*datepart("h",nexttime))
  ElseIf min mod multiple = 0 Then
    nexttime = timeserial(hr,min+offset,0)
    sec2 = datepart("s",nexttime) + (60*datepart("n",nexttime)) + (3600*datepart("h",nexttime))
  Else
    remainder = (min mod multiple)
    nexttime = timeserial(hr,min+multiple-remainder+offset,0)
    sec2 = datepart("s",nexttime) + (60*datepart("n",nexttime)) + (3600*datepart("h",nexttime))
  end if

  syncsecs = sec2 - sec1

  If syncsecs < 0 Then
    'fixes the case where start & end times on opposite sides of midnight
    syncsecs = syncsecs + 86400
    Call log(objGlobal.logpath,"Next sync time is after midnight (in " & syncsecs & " seconds).")
  End If
end function 'syncsecs()




'*********************************************************************
'** sub initialize() *************************************************
'*********************************************************************
' initializes connection between TheSky and the Paramount ME
' parks, then homes
' requires the following before calling:
'   Dim objTele
'   Set objTele = WScript.CreateObject("TheSky6.RASCOMTele")
' requires the 'log(path,msg)' subroutine
' requires the 'objGlobal.logpath' global parameter

Sub initialize()
  Call log(objGlobal.logpath,"Begin initialization.")
  Call log(objGlobal.logpath,"Attempting to connect to the mount.")
  objTele.Connect()

  If (objTele.IsConnected = 0) Then
    Call log(objGlobal.logpath,"Connection to the mount failed.")
    WScript.Quit
  Else
    Call log(objGlobal.logpath,"Successfully connected to mount.")
  End If

  Call log(objGlobal.logpath,"Initiate park.")  
  Call objTele.park()
  Call log(objGlobal.logpath,"Finished parking the mount.")
  Call log(objGlobal.logpath,"Attempting to disconnect from the mount.")
  objTele.disconnect()
  If (objTele.IsConnected = 0) Then
    Call log(objGlobal.logpath,"Successfully disconnnected from mount.")
  Else
    Call log(objGlobal.logpath,"ERROR: Mount disconnect failed.")
  End If

  Call log(objGlobal.logpath,"Attempting to reconnect to the mount.")
  objTele.Connect()

  If (objTele.IsConnected = 0) Then
    Call log(objGlobal.logpath,"Reconnection to the mount failed.")
    WScript.Quit
  Else
    Call log(objGlobal.logpath,"Successfully reconnected to mount.")
  End If

  Call log(objGlobal.logpath,"Initiate mount homing.")
  Call objTele.findhome()
  Call log(objGlobal.logpath,"Finished homing the mount.")
  Call log(objGlobal.logpath,"Finished mount initialization.")
  Call log(objGlobal.logpath,"")
end sub 'initialize()




'#####################################################################
'## Class globals ####################################################
'#####################################################################
' manages global variables

Class globals
  public cmdpath, timezone, syncmultiple
  public zoamin, zoamax, longitude, haminlim, hamaxlim
  public decminlim, decmaxlim, authpath, logpath, slewtime
  public stowha, stowdec, rewindtime, doychangetime, durationfudge

  public sub printToLog()
    Call log(objGlobal.logpath,"Global variables:")
    Call log(objGlobal.logpath,"  cmdpath: " & cmdpath)
    Call log(objGlobal.logpath,"  timezone: " & timezone)
    Call log(objGlobal.logpath,"  syncmultiple: " & syncmultiple)
    Call log(objGlobal.logpath,"  zoamin: " & zoamin)
    Call log(objGlobal.logpath,"  zoamax: " & zoamax)
    Call log(objGlobal.logpath,"  longitude: " & longitude)
    Call log(objGlobal.logpath,"  haminlim: " & haminlim)
    Call log(objGlobal.logpath,"  hamaxlim: " & hamaxlim)
    Call log(objGlobal.logpath,"  decminlim: " & decminlim)
    Call log(objGlobal.logpath,"  decmaxlim: " & decmaxlim)
    Call log(objGlobal.logpath,"  authpath: " & authpath)
    Call log(objGlobal.logpath,"  logpath: " & logpath)
    Call log(objGlobal.logpath,"  slewtime: " & slewtime)
    Call log(objGlobal.logpath,"  stowha: " & stowha)
    Call log(objGlobal.logpath,"  stowdec: " & stowdec)
    Call log(objGlobal.logpath,"  rewindtime: " & rewindtime)
    Call log(objGlobal.logpath,"  doychangetime: " & doychangetime)
Call log(objGlobal.logpath,"  durationfudge: " & durationfudge)
  End Sub 'printToLog
  
  public sub load(globals_path)
    dim filesys, readfile
    set filesys = CreateObject("Scripting.FileSystemObject")
    
    If filesys.FileExists(globals_path) Then
      set readfile = filesys.OpenTextFile(globals_path)
      
      If readfile.AtEndOfStream Then
        readfile.close()
        msgbox("ERROR: Globals file is EMPTY: " & globals_path)
        wScript.Quit
      Else
        index = 0
        do while readfile.AtEndOfStream eqv FALSE
          line = readfile.ReadLine()
          words = Split(line,"=")
          Select Case words(0)
            Case "cmdpath"
              cmdpath = words(1)
            Case "longitude"
              longitude = CDbl(words(1))
            Case "timezone"
              timezone = CInt(words(1))
            Case "syncmultiple"
              syncmultiple = CInt(words(1))
            Case "zoamin"
              zoamin = CDbl(words(1))
            Case "zoamax"
              zoamax = CDbl(words(1))
            Case "haminlim"
              haminlim = CDbl(words(1))
            Case "hamaxlim"
              hamaxlim = CDbl(words(1))
            Case "decminlim"
              decminlim = CDbl(words(1))
            Case "decmaxlim"
              decmaxlim = CDbl(words(1))
            Case "authpath"
              authpath = words(1)
            Case "logpath"
              logpath = words(1)
            Case "slewtime"
              slewtime = CInt(words(1))
            Case "rewindtime"
              rewindtime = CInt(words(1))
            Case "stowha"
              stowha = CDbl(words(1))
            Case "stowdec"
              stowdec = CDbl(words(1))
            Case "doychangetime"
              doychangetime = CDate(words(1))
   Case "durationfudge"
              durationfudge = CInt(words(1))
          End Select
        loop
        readfile.close()
      End If
    Else
      msgbox("ERROR: Globals file does NOT exist: " & globals_path)
      wScript.Quit
    End If
  end sub 'load()
end class 'globals




'*********************************************************************
'** function auth() **************************************************
'*********************************************************************
' looks for a file at 'path'
' returns 1 if the first line converts to the integer 1
'   decimals round so 1.1 returns 1 (true), 1.9 returns 0 (false)
' returns 0 (false) if first line can't be converted to 1 or if
'   the file is empty or does not exist

function auth(path)
  dim filesys, readfile
  set filesys = CreateObject("Scripting.FileSystemObject")
  If filesys.FileExists(path) Then
    Call log(objGlobal.logpath,"Mount authorization file exists.")
    set readfile = filesys.OpenTextFile(path)
    If readfile.AtEndOfStream Then
      readfile.close()
      auth = 0
      Call log(objGlobal.logpath,"ERROR: Mount authorization file is EMPTY: " & path)
    Else
      line = readfile.ReadLine()
      If IsNumeric(line) Then
        line = CInt(line)
        If line = 1 then auth = 1 else auth = 0
        Call log(objGlobal.logpath,"Mount authorization file contains: " & line)
      Else
        auth = 0
        Call log(objGlobal.logpath,"ERROR: Mount authorization file contains INVALID data: " & line)
      End If
      readfile.close()
    End If
  Else
    auth = 0
    Call log(objGlobal.logpath,"ERROR: Mount authorization file does NOT exist: " & path)
  End If
end function 'auth()




'*********************************************************************
'** sub log(path,msg) ************************************************
'*********************************************************************
' Appends 'msg' to a log file at 'path'
' prepends date and time stamp to each line written
' new log file every observing night

sub log(path,msg)
  dim filesys, filetxt
  Set filesys = CreateObject("Scripting.FileSystemObject")
  
  doy = CInt(datepart("y",now()))
  If time < objGlobal.doychangetime Then
    doy = CInt(datepart("y",dateadd("d",-1,now())))
  End If

  If len(doy) = 1 Then
    doy = "00" & doy
  ElseIf len(doy) = 2 Then
    doy = "0" & doy
  End If
  loc = InStrRev(path,"\")
  first = left(path,loc)
  middle = datepart("yyyy",now()) & "-" & doy & "_"
  last = right(path,len(path)-loc)
  path = first & middle & last

  Set filetxt = filesys.OpenTextFile(path, 8, True)
  msg = now() & " - " & msg
  filetxt.WriteLine(msg)
  filetxt.Close
end sub 'log()




'*********************************************************************
'** sub observe() ****************************************************
'*********************************************************************

sub observe()
  Call log(objGlobal.logpath,"Begin observating session.")

  'Define array which stores commands for current observation session
  Dim commandarray(99)
  Call log(objGlobal.logpath,"Load observing strategy.")
  num_cmds = loadcommands(objGlobal.cmdpath,commandarray)
  Call log(objGlobal.logpath,"Valid pointings found for this session: " & num_cmds)

  for i=0 to num_cmds-1
    Call log(objGlobal.logpath,"  #" & i+1 & ": " & commandarray(i))
  Next
  Call log(objGlobal.logpath,"")

  'For each telescope pointing, do the following
  Call log(objGlobal.logpath,"Begin observation loop.")
  for i=0 to num_cmds-1
    Call log(objGlobal.logpath,"Pointing #" & i+1 & " of " & num_cmds & ".")
    Call log(objGlobal.logpath,"Attempt to drift scan.")
    words = Split(commandarray(i)," ")

    num_elements = 0
    for each word in words
      num_elements = num_elements+1
    next
 
    if num_elements = 7 Then
      start = CDate(words(1))
      Call log(objGlobal.logpath,"  start: " & start)
      finish = CDate(words(2))
      Call log(objGlobal.logpath,"  finish: " & finish)
      ra = CDbl(words(3))
      Call log(objGlobal.logpath,"  ra: " & ra)
      decmin = CDbl(words(4))
      Call log(objGlobal.logpath,"  decmin: " & decmin)
      decmax = CDbl(words(5))
      Call log(objGlobal.logpath,"  decmax: " & decmax)
      scanrate = CDbl(words(6))
      Call log(objGlobal.logpath,"  scanrate: " & scanrate)
      duration = objGlobal.durationfudge + ((3600*(decmax-decmin))/scanrate)
      Call log(objGlobal.logpath,"  duration: " & duration)
      starttime = time2datetime(start,objGlobal.doychangetime)
      starttime = dateadd("s",-(objGlobal.slewtime),starttime)
      Call log(objGlobal.logpath,"  starttime: " & starttime)
      buffersecs = (60 * objGlobal.syncmultiple) + objGlobal.slewtime
      'Call log(objGlobal.logpath,"    buffersecs: " & buffersecs)
      finaltime = time2datetime(finish,objGlobal.doychangetime)
      'Call log(objGlobal.logpath,"    finaltime: " & finaltime)
      finaltime = dateadd("s",-(buffersecs),finaltime)
      Call log(objGlobal.logpath,"  finaltime: " & finaltime)
  
      If (duration >= ((60*objGlobal.syncmultiple)-objGlobal.rewindtime)) Then
        Call log(objGlobal.logpath,"WARNING: Scan duration long compared to sync multiple.")
        Call log(objGlobal.logpath,"  duration: " & duration & ", syncmultiple: " & objGlobal.syncmultiple &", rewindtime: " & objGlobal.rewindtime)
      End if

      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      Call objtele.settracking(1,0,0,0)

      If start < objGlobal.doychangetime and finish > objGlobal.doychangetime Then
        Call log(objGlobal.logpath,"ERROR: Cannot attempt to observe during system reboot.")'
      ElseIf starttime > finaltime Then
        Call log(objGlobal.logpath,"ERROR: starttime > finaltime.")
      Else
        If now() < starttime and now() < finaltime Then
          Call log(objGlobal.logpath,"Waiting until " & starttime)
          do while now() < starttime
            'wait
          loop
          Call log(objGlobal.logpath,"Done waiting.")
        End If

        do while now() >= starttime and now() < finaltime
          'Perform a drift scan in declination only
          continueDO = true
          Call log(objGlobal.logpath,"")
          continueDO = decscan(ra,decmin,decmax,scanrate,duration)

          if continueDO eqv false Then
            Call log(objGlobal.logpath,"ERROR: Moving on to next pointing.")
            Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
            Call objtele.settracking(1,0,0,0)
            exit do
          End If
        loop
      End If

      Call log(objGlobal.logpath,"Set tracking to sidereal rate.")
      Call objtele.settracking(1,0,0,0)
      Call log(objGlobal.logpath,"No more drift scans at pointing # " & i+1 & " of " & num_cmds)
      Call log(objGlobal.logpath,"")
    Else
      Call log(objGlobal.logpath,"ERROR: Incorrect number of columns in strategy file (" & num_elements & ").")
    End if 'strategy file format check

  next
  Call log(objGlobal.logpath,"End of observing session (no more pointings).")
  Call log(objGlobal.logpath,"")
end sub 'observe()




'*********************************************************************
'** sub stow() *******************************************************
'*********************************************************************
' requires lst(), julian(), and ha2ra() to calculate RA of stow position

sub stow()
  'stowra = lst(objGlobal.longitude,now(),objGlobal.timezone) - (objGlobal.stowha)
  stowra = ha2ra(lst(objGlobal.longitude,now(),objGlobal.timezone), objGlobal.stowha)
  Call log(objGlobal.logpath,"Initiate mount homing.")
  objTele.FindHome()
  Call log(objGlobal.logpath,"Mount homing complete.")
  'Call log(objGlobal.logpath,"Stowing at HA=" & objGlobal.stowha & ", Dec=" & objGlobal.stowdec & ". (RA=" & stowra & ")")
  'Call objtele.slewtoradec(stowra,objGlobal.stowdec,"south")
  'Call log(objGlobal.logpath,"Finished slew to stow position.")
  objTele.park()
  Call log(objGlobal.logpath,"Mount parking complete.")
  objTele.disconnect()
  If (objTele.IsConnected = 0) Then
    Call log(objGlobal.logpath,"Successfully disconnnected from mount.")
  Else
    Call log(objGlobal.logpath,"ERROR: Mount disconnect failed.")
  End If
  Call log(objGlobal.logpath,"Finished stowing the mount.")
  Call log(objGlobal.logpath,"### End of script.")
end sub 'stow()




'*********************************************************************
'** sub time2datetime() **********************************************
'*********************************************************************
' Converts a timestamp into a datetime stamp
' Decides what date to use based on current time compared with
'   the changetime

function time2datetime(intime,changetime)

  intime = CDate(intime)
  changetime = CDate(changetime)

  if intime < changetime and time < changetime Then
    time2datetime = CDate(date & " " & intime)
  elseif intime < changetime and time > changetime Then
    time2datetime = CDate(dateadd("d",1,date) & " " & intime)
  elseif intime > changetime and time < changetime Then
    time2datetime = CDate(dateadd("d",-1,date) & " " & intime)
  elseif intime > changetime and time > changetime Then
    time2datetime = CDate(date & " " & intime)
  elseif intime = changetime Then
    time2datetime = CDate(date & " " & intime)
  else
    time2datetime = CDate(date & " " & intime)
    Call log(objGlobal.logpath,"ERROR: problem converting time to datetime.")
  End If

End function 'time2datetime




'*********************************************************************
'** function ra2ha() *************************************************
'*********************************************************************

function ra2ha(lst,ra)
  ha = lst - ra
  If ha > 12.0 Then
    ha = ha - 24.0
  ElseIf ha < -12.0 Then
    ha = ha + 24.0
  End If
  ra2ha = ha
End function 'ra2ha




'*********************************************************************
'** function ha2ra() *************************************************
'*********************************************************************

function ha2ra(lst,ha)
  ra = lst - ha
  If ra >= 24.0 Then
    ra = ra - 24.0
  End If
  ha2ra = ra
End function 'ha2ra

